@@ -1,3 +1,4 @@
+from __future__ import print_function
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 '''
@@ -22,6 +23,7 @@ from calibre.gui2.progress_indicator import ProgressIndicator as _ProgressIndica
 from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
     image_extensions, dnd_has_extension, DownloadDialog)
 from calibre.utils.localization import localize_user_manual_link
+from polyglot.builtins import unicode_type, range
 
 history = XMLConfig('history')
 
@@ -82,7 +84,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         # Get all items in the combobox. If we are reseting
         # to defaults we don't want to lose what the user
         # has added.
-        val_hist = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        val_hist = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         self.re.clear()
 
         if defaults:
@@ -105,7 +107,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
     def do_test(self):
         from calibre.ebooks.metadata import authors_to_string
         from calibre.ebooks.metadata.meta import metadata_from_filename
-        fname = unicode(self.filename.text())
+        fname = unicode_type(self.filename.text())
         ext = os.path.splitext(fname)[1][1:].lower()
         if ext not in BOOK_EXTENSIONS:
             return warning_dialog(self, _('Test file name invalid'),
@@ -153,7 +155,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         self.comments.setText(mi.comments if mi.comments else _('No match'))
 
     def pattern(self):
-        pat = unicode(self.re.lineEdit().text())
+        pat = unicode_type(self.re.lineEdit().text())
         return re.compile(pat)
 
     def commit(self):
@@ -161,7 +163,7 @@ class FilenamePattern(QWidget, Ui_Form):  # {{{
         prefs['filename_pattern'] = pat
 
         history = []
-        history_pats = [unicode(self.re.lineEdit().text())] + [unicode(self.re.itemText(i)) for i in xrange(self.re.count())]
+        history_pats = [unicode_type(self.re.lineEdit().text())] + [unicode_type(self.re.itemText(i)) for i in range(self.re.count())]
         for p in history_pats[:24]:
             # Ensure we don't have duplicate items.
             if p and p not in history:
@@ -484,23 +486,23 @@ class LineEditECM(object):  # {{{
 
     def upper_case(self):
         from calibre.utils.icu import upper
-        self.setText(upper(unicode(self.text())))
+        self.setText(upper(unicode_type(self.text())))
 
     def lower_case(self):
         from calibre.utils.icu import lower
-        self.setText(lower(unicode(self.text())))
+        self.setText(lower(unicode_type(self.text())))
 
     def swap_case(self):
         from calibre.utils.icu import swapcase
-        self.setText(swapcase(unicode(self.text())))
+        self.setText(swapcase(unicode_type(self.text())))
 
     def title_case(self):
         from calibre.utils.titlecase import titlecase
-        self.setText(titlecase(unicode(self.text())))
+        self.setText(titlecase(unicode_type(self.text())))
 
     def capitalize(self):
         from calibre.utils.icu import capitalize
-        self.setText(capitalize(unicode(self.text())))
+        self.setText(capitalize(unicode_type(self.text())))
 
 # }}}
 
@@ -582,13 +584,13 @@ class CompleteLineEdit(EnLineEdit):  # {{{
         self.space_before_sep = space_before
 
     def text_changed(self, text):
-        all_text = unicode(text)
+        all_text = unicode_type(text)
         text = all_text[:self.cursorPosition()]
         prefix = text.split(self.separator)[-1].strip()
 
         text_items = []
         for t in all_text.split(self.separator):
-            t1 = unicode(t).strip()
+            t1 = unicode_type(t).strip()
             if t1 != '':
                 text_items.append(t)
         text_items = list(set(text_items))
@@ -596,8 +598,8 @@ class CompleteLineEdit(EnLineEdit):  # {{{
 
     def complete_text(self, text):
         cursor_pos = self.cursorPosition()
-        before_text = unicode(self.text())[:cursor_pos]
-        after_text = unicode(self.text())[cursor_pos:]
+        before_text = unicode_type(self.text())[:cursor_pos]
+        after_text = unicode_type(self.text())[cursor_pos:]
         prefix_len = len(before_text.split(self.separator)[-1].lstrip())
         if self.space_before_sep:
             complete_text_pat = '%s%s %s %s'
@@ -626,7 +628,7 @@ class EnComboBox(QComboBox):  # {{{
         self.setMinimumContentsLength(20)
 
     def text(self):
-        return unicode(self.currentText())
+        return unicode_type(self.currentText())
 
     def setText(self, text):
         idx = self.findText(text, Qt.MatchFixedString|Qt.MatchCaseSensitive)
@@ -682,11 +684,11 @@ class HistoryLineEdit(QComboBox):  # {{{
 
     def save_history(self):
         items = []
-        ct = unicode(self.currentText())
+        ct = unicode_type(self.currentText())
         if ct:
             items.append(ct)
         for i in range(self.count()):
-            item = unicode(self.itemText(i))
+            item = unicode_type(self.itemText(i))
             if item not in items:
                 items.append(item)
         self.blockSignals(True)
@@ -1080,7 +1082,7 @@ class Splitter(QSplitter):
                 parent.addAction(self.action_toggle)
                 if hasattr(parent, 'keyboard'):
                     parent.keyboard.register_shortcut('splitter %s %s'%(name,
-                        label), unicode(self.action_toggle.text()),
+                        label), unicode_type(self.action_toggle.text()),
                         default_keys=(shortcut,), action=self.action_toggle)
                 else:
                     self.action_toggle.setShortcut(shortcut)
@@ -1126,33 +1128,31 @@ class Splitter(QSplitter):
 
     def print_sizes(self):
         if self.count() > 1:
-            print self.save_name, 'side:', self.side_index_size, 'other:',
-            print list(self.sizes())[self.other_index]
+            print(self.save_name, 'side:', self.side_index_size, 'other:', end=' ')
+            print(list(self.sizes())[self.other_index])
 
-    @dynamic_property
+    @property
     def side_index_size(self):
-        def fget(self):
-            if self.count() < 2:
-                return 0
-            return self.sizes()[self.side_index]
+        if self.count() < 2:
+            return 0
+        return self.sizes()[self.side_index]
 
-        def fset(self, val):
-            if self.count() < 2:
-                return
-            if val == 0 and not self.is_side_index_hidden:
-                self.save_state()
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else 10
-            self.setSizes(sizes)
-            total = sum(self.sizes())
-            sizes = list(self.sizes())
-            for i in range(len(sizes)):
-                sizes[i] = val if i == self.side_index else total-val
-            self.setSizes(sizes)
-            self.initialize()
-
-        return property(fget=fget, fset=fset)
+    @side_index_size.setter
+    def side_index_size(self, val):
+        if self.count() < 2:
+            return
+        if val == 0 and not self.is_side_index_hidden:
+            self.save_state()
+        sizes = list(self.sizes())
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else 10
+        self.setSizes(sizes)
+        total = sum(self.sizes())
+        sizes = list(self.sizes())
+        for i in range(len(sizes)):
+            sizes[i] = val if i == self.side_index else total-val
+        self.setSizes(sizes)
+        self.initialize()
 
     def do_resize(self, *args):
         orig = self.desired_side_size
